# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

import numpy as np

from Octeres.random_generation import measure_random_distribution, measure_fixed_matrix_distribution
from Octeres.random_generation import measure_random_matrix_distribution


def test_random_time():
    """Run the whole process 100 times to test the performance.
    Each process would stop when the sum of intervals reach a fixed amount."""
    RandomDistribution_times = []
    RandomMatrixDistribution_times = []
    FixedMatrixDistribution_times = []

    for _ in range(100):
        RandomDistribution_times.append(measure_random_distribution())
        RandomMatrixDistribution_times.append(measure_random_matrix_distribution())
        FixedMatrixDistribution_times.append(measure_fixed_matrix_distribution())

    print(f"Average running time for RandomDistribution: {np.mean(RandomDistribution_times):.6f} seconds")
    print(f"Average running time for RandomMatrixDistribution: {np.mean(RandomMatrixDistribution_times):.6f} seconds")
    print(f"Average running time for FixedMatrixDistribution: {np.mean(FixedMatrixDistribution_times):.6f} seconds")
