# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

"""
Unit tests for util.
"""
from itertools import zip_longest
from Octeres.util import iterate_chain, poll, Timed, superprint
import pytest


def test_superprint():
    superprint("HI!")

    def a():
        superprint("IN a", jump_back_count=3)
        b()

    def b():
        superprint("IN b", jump_back_count=3)
        c()

    def c():
        superprint("IN c", jump_back_count=3)
        d()

    def d():
        superprint("IN d", jump_back_count=3)
        pass

    superprint("calling a()")
    a()

    # FIXME: need to capture stdout and assert that it's what we expect
    # assert 1 == 2


def test_iterate_chain_00():
    result = iterate_chain(["a", "b", "c", "d", "e"])
    result = list(result)
    correct = [
        (None, "a", "b"),
        ("a", "b", "c"),
        ("b", "c", "d"),
        ("c", "d", "e"),
        ("d", "e", None),
    ]
    superprint(result)
    assert result == correct


def test_poll_00():
    dev = 0.01
    loopcount = 4
    sleeploop = 0.04
    td = Timed()
    with td.timer("test") as timer:
        for sleeptime, first in zip_longest(poll(loopcount, sleeploop), [True], fillvalue=False):
            if first:
                continue  # warmup, always 0.0
            assert (sleeploop - dev) < sleeptime, sleeptime
            assert sleeptime < (sleeploop + dev), sleeptime

    assert (loopcount - 1) * (sleeploop - dev) < timer.td, timer.td
    assert timer.td < (loopcount - 1) * (sleeploop + dev), timer.td


def test_poll_01():
    dev = 0.01
    loopcount = 4
    sleeploop = 0.04
    td = Timed()
    with td.timer("test", use_gc=False) as timer:
        for sleeptime, first in zip_longest(poll(loopcount, sleeploop), [True], fillvalue=False):
            if first:
                continue  # warmup, always 0.0
            assert (sleeploop - dev) < sleeptime, sleeptime
            assert sleeptime < (sleeploop + dev), sleeptime

    assert (loopcount - 1) * (sleeploop - dev) < timer.td, timer.td
    assert timer.td < (loopcount - 1) * (sleeploop + dev), timer.td
